# -*- coding: utf-8 -*-
## $Id: interfaces.py 17370 2023-03-29 13:15:45Z henrik $
##
## Copyright (C) 2011 Uli Fouquet & Henrik Bettermann
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 2 of the License, or
## (at your option) any later version.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with this program; if not, write to the Free Software
## Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
##
"""Customized interfaces of the university application package.
"""

from zope import schema
from zope.interface import invariant, Invalid
from zope.component import getUtility
from zope.catalog.interfaces import ICatalog
from zc.sourcefactory.basic import BasicSourceFactory
from waeup.kofa.applicants.interfaces import (
    IApplicantBaseData,
    AppCatCertificateSource, CertificateSource)
from waeup.kofa.schoolgrades import ResultEntryField
from waeup.kofa.interfaces import (
    SimpleKofaVocabulary, IKofaObject,
    academic_sessions_vocab, validate_email)
from waeup.kofa.university.vocabularies import StudyModeSource
from waeup.kofa.schema import FormattedDate, TextLineChoice, PhoneNumber
from waeup.kofa.students.vocabularies import nats_vocab, GenderSource
from waeup.kofa.applicants.interfaces import (
    contextual_reg_num_source, ISpecialApplicant)
from kofacustom.nigeria.applicants.interfaces import (
    LGASource, high_qual, high_grade, exam_types,
    INigeriaUGApplicant, INigeriaPGApplicant,
    INigeriaApplicantOnlinePayment,
    INigeriaUGApplicantEdit, INigeriaPGApplicantEdit,
    INigeriaApplicantUpdateByRegNo,
    IPUTMEApplicantEdit,
    )
from kofacustom.edopoly.interfaces import MessageFactory as _
from kofacustom.edopoly.payments.interfaces import ICustomOnlinePayment

class TranscriptCertificateSource(CertificateSource):
    """Include Department and Faculty in Title.
    """
    def getValues(self, context):
        catalog = getUtility(ICatalog, name='certificates_catalog')
        resultset = catalog.searchResults(code=(None, None))
        resultlist = sorted(resultset, key=lambda
            value: value.__parent__.__parent__.__parent__.code +
            value.__parent__.__parent__.code +
            value.code)
        return resultlist

    def getTitle(self, context, value):
        """
        """
        try: title = "%s / %s / %s (%s)" % (
            value.__parent__.__parent__.__parent__.title,
            value.__parent__.__parent__.title,
            value.title, value.code)
        except AttributeError:
            title = "NA / %s (%s)" % (value.title, value.code)
        return title

DESTINATION_COST = {
    #'none': ('To the moon', 1000000000.0, 1),
    'nigeria': ('Within Nigeria', 20000.0, 1),
    'africa': ('Within Africa ', 30000.0, 2),
    'inter': ('International', 35000.0, 3),
    'cert_nigeria': ('Certified Copy - Within Nigeria', 13000.0, 4),
    'cert_africa': ('Certified Copy - Within Africa', 23000.0, 5),
    'cert_inter': ('Certified Copy - International', 28000.0, 6),
    }

class DestinationCostSource(BasicSourceFactory):
    """A source that delivers continents and shipment costs.
    """
    def getValues(self):
        sorted_items = sorted(DESTINATION_COST.items(),
                              key=lambda element: element[1][2])
        return [item[0] for item in sorted_items]

    def getToken(self, value):
        return value

    def getTitle(self, value):
        return u"%s (₦ %s)" % (
            DESTINATION_COST[value][0],
            DESTINATION_COST[value][1])

class OrderSource(BasicSourceFactory):
    """
    """
    def getValues(self):
        return ['o', 'c']

    def getToken(self, value):
        return value

    def getTitle(self, value):
        if value == 'o':
            return _('Original')
        if value == 'c':
            return _('Certified True Copy')

class ICustomUGApplicant(IApplicantBaseData):
    """An undergraduate applicant.

    This interface defines the least common multiple of all fields
    in ug application forms. In customized forms, fields can be excluded by
    adding them to the OMIT* tuples.
    """

    nationality = schema.Choice(
        source = nats_vocab,
        title = _(u'Nationality'),
        required = True,
        )
    lga = schema.Choice(
        source = LGASource(),
        title = _(u'State/LGA (Nigerians only)'),
        required = False,
        )
    #perm_address = schema.Text(
    #    title = _(u'Permanent Address'),
    #    required = False,
    #    )
    course1 = schema.Choice(
        title = _(u'1st Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = False,
        )
    course2 = schema.Choice(
        title = _(u'2nd Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = False,
        )
    olevel_type = schema.Choice(
        title = _(u'1st Qualification Obtained'),
        required = False,
        readonly = False,
        vocabulary = exam_types,
        )
    olevel_school = schema.TextLine(
        title = _(u'1st Institution Attended'),
        required = False,
        readonly = False,
        )
    olevel_exam_number = schema.TextLine(
        title = _(u'1st Exam Number'),
        required = False,
        readonly = False,
        )
    olevel_exam_date = FormattedDate(
        title = _(u'1st Exam Date'),
        required = False,
        readonly = False,
        show_year = True,
        )
    olevel_results = schema.List(
        title = _(u'1st Exam Results'),
        value_type = ResultEntryField(),
        required = False,
        readonly = False,
        defaultFactory=list,
        )
    olevel_type2 = schema.Choice(
        title = _(u'2nd Qualification Obtained'),
        required = False,
        readonly = False,
        vocabulary = exam_types,
        )
    olevel_school2 = schema.TextLine(
        title = _(u'2nd Institution Attended'),
        required = False,
        readonly = False,
        )
    olevel_exam_number2 = schema.TextLine(
        title = _(u'2nd Exam Number'),
        required = False,
        readonly = False,
        )
    olevel_exam_date2 = FormattedDate(
        title = _(u'2nd Exam Date'),
        required = False,
        readonly = False,
        show_year = True,
        )
    olevel_results2 = schema.List(
        title = _(u'2nd Exam Results'),
        value_type = ResultEntryField(),
        required = False,
        readonly = False,
        defaultFactory=list,
        )
    hq_type = schema.Choice(
        title = _(u'Qualification Obtained'),
        required = False,
        readonly = False,
        vocabulary = high_qual,
        )
    hq_matric_no = schema.TextLine(
        title = _(u'Former Matric Number'),
        required = False,
        readonly = False,
        )
    hq_degree = schema.Choice(
        title = _(u'Class of Degree'),
        required = False,
        readonly = False,
        vocabulary = high_grade,
        )
    hq_school = schema.TextLine(
        title = _(u'Institution Attended'),
        required = False,
        readonly = False,
        )
    hq_session = schema.TextLine(
        title = _(u'Years Attended'),
        required = False,
        readonly = False,
        )
    hq_disc = schema.TextLine(
        title = _(u'Discipline'),
        required = False,
        readonly = False,
        )
    jamb_subjects = schema.Text(
        title = _(u'Subjects and Scores'),
        description = _(u'(one subject with score per line)'),
        required = False,
        )
    jamb_score = schema.Int(
        title = _(u'Total JAMB Score'),
        required = False,
        )
    jamb_reg_number = schema.TextLine(
        title = _(u'JAMB Registration Number'),
        required = False,
        )
    notice = schema.Text(
        title = _(u'Notice'),
        required = False,
        )
    screening_venue = schema.TextLine(
        title = _(u'Screening Venue'),
        required = False,
        )
    screening_date = schema.TextLine(
        title = _(u'Screening Date'),
        required = False,
        )
    screening_score = schema.Int(
        title = _(u'Screening Score (%)'),
        required = False,
        )
    aggregate = schema.Int(
        title = _(u'Aggregate Score (%)'),
        description = _(u'(average of relative JAMB and PUTME scores)'),
        required = False,
        )
    result_uploaded = schema.Bool(
        title = _(u'Result uploaded'),
        default = False,
        required = False,
        )
    student_id = schema.TextLine(
        title = _(u'Student Id'),
        required = False,
        readonly = False,
        )
    course_admitted = schema.Choice(
        title = _(u'Admitted Course of Study'),
        source = CertificateSource(),
        required = False,
        )
    locked = schema.Bool(
        title = _(u'Form locked'),
        default = False,
        required = False,
        )

class ITranscriptApplicant(IKofaObject):
    """A transcript applicant.
    """

    suspended = schema.Bool(
        title = _(u'Account suspended'),
        default = False,
        required = False,
        )

    locked = schema.Bool(
        title = _(u'Form locked'),
        default = False,
        required = False,
        )

    applicant_id = schema.TextLine(
        title = _(u'Transcript Application Id'),
        required = False,
        readonly = False,
        )

    student_id = schema.TextLine(
        title = _(u'Kofa Student Id'),
        required = False,
        readonly = False,
        )

    matric_number = schema.TextLine(
        title = _(u'Matriculation Number'),
        readonly = False,
        required = True,
        )

    firstname = schema.TextLine(
        title = _(u'First Name in School'),
        required = True,
        )

    middlename = schema.TextLine(
        title = _(u'Middle Name in School'),
        required = False,
        )

    lastname = schema.TextLine(
        title = _(u'Surname in School'),
        required = True,
        )

    date_of_birth = FormattedDate(
        title = _(u'Date of Birth'),
        required = False,
        #date_format = u'%d/%m/%Y', # Use grok-instance-wide default
        show_year = True,
        )

    sex = schema.Choice(
        title = _(u'Gender'),
        source = GenderSource(),
        required = True,
        )

    #nationality = schema.Choice(
    #    vocabulary = nats_vocab,
    #    title = _(u'Nationality'),
    #    required = False,
    #    )

    email = schema.ASCIILine(
        title = _(u'Email Address'),
        required = True,
        constraint=validate_email,
        )

    phone = PhoneNumber(
        title = _(u'Phone'),
        description = u'',
        required = False,
        )

    #perm_address = schema.Text(
    #    title = _(u'Current Local Address'),
    #    required = False,
    #    readonly = False,
    #    )

    collected = schema.Bool(
        title = _(u'Have you collected transcript before?'),
        default = False,
        required = False,
        )

    entry_session = schema.Choice(
        title = _(u'Academic Session of Entry'),
        source = academic_sessions_vocab,
        required = False,
        readonly = False,
        )

    end_session = schema.Choice(
        title = _(u'Academic Session of Graduation'),
        source = academic_sessions_vocab,
        required = False,
        readonly = False,
        )

    entry_mode = schema.Choice(
        title = _(u'Mode of Entry'),
        source = StudyModeSource(),
        required = False,
        readonly = False,
        )

    course_studied = schema.Choice(
        title = _(u'Course of Study'),
        source = TranscriptCertificateSource(),
        description = u'Faculty / Department / Course',
        required = True,
        readonly = False,
        )

    course_changed = schema.Choice(
        title = _(u'Change of Study Course / Transfer'),
        description = u'If yes, select previous course of study.',
        source = TranscriptCertificateSource(),
        readonly = False,
        required = False,
        )

    spillover_level = schema.Choice(
        title = _(u'Spill-over'),
        description = u'Any spill-over? If yes, select session of spill-over.',
        source = academic_sessions_vocab,
        required = False,
        readonly = False,
        )

    purpose = schema.TextLine(
        title = _(u'Transcript Purpose'),
        required = False,
        )

    order = schema.Choice(
        source = OrderSource(),
        title = _(u'Type of Order'),
        required = False,
        )

    dispatch_address = schema.Text(
        title = _(u'Recipient Body'),
        description = u'Addresses (including email address and phone number) '
                       'to which transcripts should be posted. '
                       'All addresses must involve same courier charges.',
        required = True,
        readonly = False,
        )

    charge = schema.Choice(
        title = _(u'Transcript Charge'),
        source = DestinationCostSource(),
        required = False,
        readonly = False,
        )

    no_copies = schema.Choice(
        title = _(u'Number of Copies'),
        description = u'Must correspond with the number of dispatch addresses above.',
        values=[1, 2, 3, 4],
        required = False,
        readonly = False,
        default = 1,
        )

    courier_tno = schema.TextLine(
        title = _(u'Courier Tracking Number'),
        required = False,
        )

    proc_date = FormattedDate(
        title = _(u'Processing Date'),
        required = False,
        #date_format = u'%d/%m/%Y', # Use grok-instance-wide default
        show_year = True,
        )

    @invariant
    def type_of_order(applicant):
        if not applicant.collected and applicant.order != 'o':
            raise Invalid(_("If you haven't collected transcript before, type of order must be 'original'."))
        if applicant.order == 'o' and applicant.charge.startswith('cert_'):
            raise Invalid(_("You've selected the wrong transcript charge."))
        if applicant.order == 'c' and not applicant.charge.startswith('cert_'):
            raise Invalid(_("You've selected the wrong transcript charge."))

class ICustomPGApplicant(INigeriaPGApplicant):
    """A postgraduate applicant.

    This interface defines the least common multiple of all fields
    in pg application forms. In customized forms, fields can be excluded by
    adding them to the PG_OMIT* tuples.
    """

class ICustomSpecialApplicant(ISpecialApplicant):
    """
    """

class ICustomApplicant(ICustomUGApplicant, ICustomPGApplicant,
    ITranscriptApplicant):
    """An interface for both types of applicants.

    Attention: The ICustomPGApplicant field seetings will be overwritten
    by ICustomPGApplicant field settings. If a field is defined
    in both interfaces zope.schema validates only against the
    constraints in ICustomUGApplicant. This does not affect the forms
    since they are build on either ICustomUGApplicant or ICustomPGApplicant.
    """

    def writeLogMessage(view, comment):
        """Adds an INFO message to the log file
        """

    def createStudent():
        """Create a student object from applicant data
        and copy applicant object.
        """

class ICustomUGApplicantEdit(ICustomUGApplicant):
    """An undergraduate applicant interface for edit forms.

    Here we can repeat the fields from base data and set the
    `required` and `readonly` attributes to True to further restrict
    the data access. Or we can allow only certain certificates to be
    selected by choosing the appropriate source.

    We cannot omit fields here. This has to be done in the
    respective form page.
    """


    email = schema.ASCIILine(
        title = _(u'Email Address'),
        required = True,
        constraint=validate_email,
        )
    date_of_birth = FormattedDate(
        title = _(u'Date of Birth'),
        required = True,
        show_year = True,
        )
    jamb_reg_number = schema.TextLine(
        title = _(u'JAMB Registration Number'),
        required = True,
        )
    course1 = schema.Choice(
        title = _(u'1st Choice Course of Study'),
        source = AppCatCertificateSource(),
        required = True,
        )

ICustomUGApplicantEdit[
    'date_of_birth'].order =  ICustomUGApplicant['date_of_birth'].order
ICustomUGApplicantEdit[
    'email'].order =  ICustomUGApplicant['email'].order
ICustomUGApplicantEdit[
    'jamb_reg_number'].order =  ICustomUGApplicant['jamb_reg_number'].order
ICustomUGApplicantEdit[
    'course1'].order =  ICustomUGApplicant['course1'].order

class ICustomPGApplicantEdit(ICustomPGApplicant):
    """A postgraduate applicant interface for editing.

    Here we can repeat the fields from base data and set the
    `required` and `readonly` attributes to True to further restrict
    the data access. Or we can allow only certain certificates to be
    selected by choosing the appropriate source.

    We cannot omit fields here. This has to be done in the
    respective form page.
    """

class ICustomApplicantOnlinePayment(INigeriaApplicantOnlinePayment):
    """An applicant payment via payment gateways.

    """

class IPUTMEApplicantEdit(IPUTMEApplicantEdit):
    """An undergraduate applicant interface for editing.

    Here we can repeat the fields from base data and set the
    `required` and `readonly` attributes to True to further restrict
    the data access. Or we can allow only certain certificates to be
    selected by choosing the appropriate source.

    We cannot omit fields here. This has to be done in the
    respective form page.
    """

class ICustomApplicantUpdateByRegNo(INigeriaApplicantUpdateByRegNo):
    """Representation of an applicant.

    Skip regular reg_number validation if reg_number is used for finding
    the applicant object.
    """
